@@ -27,6 +27,7 @@ import pkgutil
 import importlib.util
 import time
 import threading
+import sys
 from typing import NamedTuple, Any, Union, TYPE_CHECKING, Optional
 
 from .i18n import _
@@ -73,6 +74,9 @@ class Plugins(DaemonThread):
                 raise Exception(f"Error pre-loading {full_name}: no spec")
             try:
                 module = importlib.util.module_from_spec(spec)
+                # sys.modules needs to be modified for relative imports to work
+                # see https://stackoverflow.com/a/50395128
+                sys.modules[spec.name] = module
                 spec.loader.exec_module(module)
             except Exception as e:
                 raise Exception(f"Error pre-loading {full_name}: {repr(e)}") from e
@@ -283,6 +287,7 @@ class BasePlugin(Logger):
 
 class DeviceNotFoundError(Exception): pass
 class DeviceUnpairableError(Exception): pass
+class HardwarePluginLibraryUnavailable(Exception): pass
 
 
 class Device(NamedTuple):
@@ -402,7 +407,7 @@ class DeviceMgr(ThreadJob):
 
     def unpair_xpub(self, xpub):
         with self.lock:
-            if not xpub in self.xpub_ids:
+            if xpub not in self.xpub_ids:
                 return
             _id = self.xpub_ids.pop(xpub)
             self._close_client(_id)
@@ -502,7 +507,7 @@ class DeviceMgr(ThreadJob):
         unpaired device accepted by the plugin.'''
         if not plugin.libraries_available:
             message = plugin.get_library_not_available_message()
-            raise Exception(message)
+            raise HardwarePluginLibraryUnavailable(message)
         if devices is None:
             devices = self.scan_devices()
         devices = [dev for dev in devices if not self.xpub_by_id(dev.id_)]
