@@ -10,8 +10,8 @@ from .digitalbitbox import DigitalBitboxPlugin
 
 
 class Plugin(DigitalBitboxPlugin, QtPluginBase):
-    icon_unpaired = ":icons/digitalbitbox_unpaired.png"
-    icon_paired = ":icons/digitalbitbox.png"
+    icon_unpaired = "digitalbitbox_unpaired.png"
+    icon_paired = "digitalbitbox.png"
 
     def create_handler(self, window):
         return DigitalBitbox_Handler(window)
