@@ -4,7 +4,7 @@ import sys
 
 from electrum.util import bfh, bh2u, UserCancelled, UserFacingException
 from electrum.bitcoin import TYPE_ADDRESS, TYPE_SCRIPT
-from electrum.bip32 import deserialize_xpub
+from electrum.bip32 import BIP32Node
 from electrum import constants
 from electrum.i18n import _
 from electrum.transaction import deserialize, Transaction
@@ -85,10 +85,12 @@ class KeepKeyPlugin(HW_PluginBase):
             import keepkeylib
             import keepkeylib.ckd_public
             import keepkeylib.transport_hid
+            import keepkeylib.transport_webusb
             self.client_class = client.KeepKeyClient
             self.ckd_public = keepkeylib.ckd_public
             self.types = keepkeylib.client.types
-            self.DEVICE_IDS = keepkeylib.transport_hid.DEVICE_IDS
+            self.DEVICE_IDS = (keepkeylib.transport_hid.DEVICE_IDS +
+                               keepkeylib.transport_webusb.DEVICE_IDS)
             self.device_manager().register_devices(self.DEVICE_IDS)
             self.libraries_available = True
         except ImportError:
@@ -98,8 +100,15 @@ class KeepKeyPlugin(HW_PluginBase):
         from keepkeylib.transport_hid import HidTransport
         return HidTransport(pair)
 
+    def webusb_transport(self, device):
+        from keepkeylib.transport_webusb import WebUsbTransport
+        for d in WebUsbTransport.enumerate():
+            if device.id_.startswith(d.getSerialNumber()):
+                return WebUsbTransport(d)
+        return WebUsbTransport(device)
+
     def _try_hid(self, device):
-        self.print_error("Trying to connect over USB...")
+        self.logger.info("Trying to connect over USB...")
         if device.interface_number == 1:
             pair = [None, device.path]
         else:
@@ -110,16 +119,28 @@ class KeepKeyPlugin(HW_PluginBase):
         except BaseException as e:
             # see fdb810ba622dc7dbe1259cbafb5b28e19d2ab114
             # raise
-            self.print_error("cannot connect at", device.path, str(e))
+            self.logger.info(f"cannot connect at {device.path} {e}")
+            return None
+
+    def _try_webusb(self, device):
+        self.logger.info("Trying to connect over WebUSB...")
+        try:
+            return self.webusb_transport(device)
+        except BaseException as e:
+            self.logger.info(f"cannot connect at {device.path} {e}")
             return None
 
     def create_client(self, device, handler):
-        transport = self._try_hid(device)
+        if device.product_key[1] == 2:
+            transport = self._try_webusb(device)
+        else:
+            transport = self._try_hid(device)
+
         if not transport:
-            self.print_error("cannot connect to device")
+            self.logger.info("cannot connect to device")
             return
 
-        self.print_error("connected to device at", device.path)
+        self.logger.info(f"connected to device at {device.path}")
 
         client = self.client_class(transport, handler, self)
 
@@ -127,14 +148,14 @@ class KeepKeyPlugin(HW_PluginBase):
         try:
             client.ping('t')
         except BaseException as e:
-            self.print_error("ping failed", str(e))
+            self.logger.info(f"ping failed {e}")
             return None
 
         if not client.atleast_version(*self.minimum_firmware):
             msg = (_('Outdated {} firmware for device labelled {}. Please '
                      'download the updated firmware from {}')
                    .format(self.device, client.label(), self.firmware_URL))
-            self.print_error(msg)
+            self.logger.info(msg)
             if handler:
                 handler.show_error(msg)
             else:
@@ -194,7 +215,7 @@ class KeepKeyPlugin(HW_PluginBase):
         except UserCancelled:
             exit_code = 1
         except BaseException as e:
-            traceback.print_exc(file=sys.stderr)
+            self.logger.exception('')
             handler.show_error(str(e))
             exit_code = 1
         finally:
@@ -206,6 +227,8 @@ class KeepKeyPlugin(HW_PluginBase):
         language = 'english'
         devmgr = self.device_manager()
         client = devmgr.client_by_id(device_id)
+        if not client:
+            raise Exception(_("The device was disconnected."))
 
         if method == TIM_NEW:
             strength = 64 * (item + 2)  # 128, 192 or 256
@@ -227,13 +250,13 @@ class KeepKeyPlugin(HW_PluginBase):
                                        label, language)
 
     def _make_node_path(self, xpub, address_n):
-        _, depth, fingerprint, child_num, chain_code, key = deserialize_xpub(xpub)
+        bip32node = BIP32Node.from_xkey(xpub)
         node = self.types.HDNodeType(
-            depth=depth,
-            fingerprint=int.from_bytes(fingerprint, 'big'),
-            child_num=int.from_bytes(child_num, 'big'),
-            chain_code=chain_code,
-            public_key=key,
+            depth=bip32node.depth,
+            fingerprint=int.from_bytes(bip32node.fingerprint, 'big'),
+            child_num=int.from_bytes(bip32node.child_number, 'big'),
+            chain_code=bip32node.chaincode,
+            public_key=bip32node.eckey.get_public_key_bytes(compressed=True),
         )
         return self.types.HDNodePathType(node=node, address_n=address_n)
 
