@@ -1,9 +1,12 @@
-#from btchip.btchipPersoWizard import StartBTChipPersoDialog
+from functools import partial
+
+from PyQt5.QtCore import pyqtSignal
+from PyQt5.QtWidgets import QInputDialog, QLabel, QVBoxLayout, QLineEdit
 
 from electrum.i18n import _
 from electrum.plugin import hook
 from electrum.wallet import Standard_Wallet
-from electrum.gui.qt.util import *
+from electrum.gui.qt.util import WindowModalDialog
 
 from .ledger import LedgerPlugin
 from ..hw_wallet.qt import QtHandlerBase, QtPluginBase
@@ -11,8 +14,8 @@ from ..hw_wallet.plugin import only_hook_if_libraries_available
 
 
 class Plugin(LedgerPlugin, QtPluginBase):
-    icon_unpaired = ":icons/ledger_unpaired.png"
-    icon_paired = ":icons/ledger.png"
+    icon_unpaired = "ledger_unpaired.png"
+    icon_paired = "ledger.png"
 
     def create_handler(self, window):
         return Ledger_Handler(window)
@@ -78,6 +81,3 @@ class Ledger_Handler(QtHandlerBase):
         
     def setup_dialog(self):
         self.show_error(_('Initialization of Ledger HW devices is currently disabled.'))
-        return
-        dialog = StartBTChipPersoDialog()
-        dialog.exec_()
